import pathlib
import argparse
import tarfile
import os
import sys


def process_files(project_name, path_in, path_out):
    # Make dirs
    for d in pathlib.Path(path_in).glob('**/*'):
        if d.is_dir():
            dir_out = path_out + str(d).replace(path_in, '')
            L = len(str(path_out))
            dir_out = dir_out[0:L] + \
                dir_out[L:].replace('helloml', project_name)
            pathlib.Path(dir_out).mkdir(parents=True, exist_ok=True)

    # Process Files
    for f in pathlib.Path(path_in).glob('**/*'):
        if f.is_file():
            filename_in = str(f)

            filename_out = path_out + filename_in.replace(path_in, '')
            L = len(str(path_out))
            filename_out = filename_out[0:L] + \
                filename_out[L:].replace('helloml', project_name)

            if filename_in.find('.png') == -1:
                with open(filename_in, 'r') as fin:
                    fcontent = fin.read()
                    if fcontent.find('helloml') != -1:
                        fcontent = fcontent.replace('helloml', project_name)

                with open(filename_out, 'w') as fout:
                    fout.write(fcontent)
            else:
                with open(filename_in, 'rb') as fin:
                    fcontent = fin.read()

                with open(filename_out, 'wb') as fout:
                    fout.write(fcontent)


parser = argparse.ArgumentParser(
    prog='black-box-ops',
    description='Auto generate a fully functional Machine Learning pipeline'
)
parser.add_argument(
    '--name',
    required=True,
    type=str,
    nargs=1,
    help='name of the model to be distributed'
)

parser.add_argument(
    '--noflow',
    action='store_true',
    help='exclude flow component. Typically, for deterministic (non-trainable) models'
)

if __name__ == "__main__":
    curr_path = pathlib.Path(__file__).parent.absolute()

    args = parser.parse_args()

    add_flow = True
    if args.noflow:
        add_flow = False

    path_in_list = [
        'base'
    ]
    if add_flow:
        path_in_list.append('flow')

    for pi in path_in_list:
        path_in = str(curr_path) + '/lib/{}_templates/'.format(pi)
        tarfile_in = str(curr_path) + '/lib/{}_templates.tar.gz'.format(pi)

        with tarfile.open(tarfile_in) as tar:
            tar.extractall(path_in)

        PROJECT_NAME = str(args.name[0])

        process_files(
            project_name=PROJECT_NAME,
            path_in=path_in,
            path_out=str(pathlib.Path().cwd()) + '/'
            # path_out=str(pathlib.Path().cwd()) + '/temp_out/'
        )

        os.system("rm -rf {}".format(path_in))
