from flow.mutable_model import MutableModel
import flow.pipeline_data as dp
import numpy as np

# Initialize Model (Mutable subclass)
mmdl = MutableModel()

# Fetch data to be used for training purposes
data = dp.get_data()
splitted_data = dp.split_data(data)

# Train model
mmdl.fit(splitted_data['X_train'], splitted_data['y_train'])
# y_pred = mmdl.predict(splitted_data['X_test'])
y_pred = mmdl.execute(
    method_name='predict',
    X=splitted_data['X_test']
)

mmdl.set_performance(
    y_true=splitted_data['y_test'],
    y_pred=y_pred
)

# Add extra artifacts
# -- Data Logs
# mmdl.log_extra(
#     key='test_rows',
#     value=splitted_data['X_test'].shape[0]
# )
# mmdl.log_extra(
#     key='train_rows',
#     value=splitted_data['X_train'].shape[0]
# )
# -- More metrics
from sklearn.metrics import mean_absolute_error
mmdl.log_extra(
    key='mean_absolute_error', 
    value=mean_absolute_error(
        y_true=splitted_data['y_test'], 
        y_pred=y_pred)
    )

from sklearn.metrics import r2_score
mmdl.log_extra(
    key='r2_score', 
    value=r2_score(
        y_true=splitted_data['y_test'], 
        y_pred=y_pred)
    )
# - Training Message
msg = '__train.py'
user = 'unknown'
import os
osenv = os.environ
if 'USER' in osenv.keys():
    user = osenv['USER']
msg += '/' + user

# - SAVE
mmdl.save_trained(message=msg)
