import pathlib
import joblib
import uuid
import datetime

from helloml.architecture import backend_model, \
    backend_performance

class BookKeeper(object):
    def __init__(self):
        self.backend_path = str(pathlib.Path(
            __file__).parent.absolute()) + '/.bin'
        pathlib.Path(self.backend_path).mkdir(parents=True, exist_ok=True)

        self.backend_file = self.backend_path + '/backend.joblib'

        self.model_backend_getter = backend_model
        self.performance_backend_getter = backend_performance

    def put_backend(self, backend):
        joblib.dump(backend, self.backend_file, compress=3)

    def get_backend(self):
        backend = {
            'model': None,
            'performance': None,
            'uuid': None,
            'created_at': None,
            'trained_at': None,
            'message': ''
        }
        if pathlib.Path(self.backend_file).is_file():
            backend = joblib.load(self.backend_file)
        else:
            backend['model'] = self.model_backend_getter()
            backend['performance'] = self.performance_backend_getter()
            backend['created_at'] = datetime.datetime.utcnow()

            self.put_backend(backend)
        return(backend)
