# Performance
# object in charge of keeping track for
# main metric used to measure the performance
# of backend model.

# Default Performance doesn't do anything
def not_metric(*args, **kwargs):
    return(0.0)

class Performance(object):
    def __init__(self,
                 metric_function=None,
                 abreviation: str = None,
                 min_value: float = None,
                 max_value: float = None,
                 is_bigger_better: bool = None,
                 version_length=4):
        if metric_function is None:
            assert abreviation is None
            assert min_value is None
            assert max_value is None
            assert is_bigger_better is None

            self.metric_function = not_metric
            self.abreviation = 'not'
            self.name = 'not_metric'
            self.min_value = 0.0
            self.max_value = 1.0
            self.current_value = 0.0
            self.is_bigger_better = True
        else:
            self.metric_function = metric_function
            self.name = metric_function.__name__
            assert len(abreviation) == 3
            self.abreviation = abreviation
            assert min_value < max_value
            self.min_value = min_value
            self.max_value = max_value
            self.is_bigger_better = is_bigger_better
            if self.is_bigger_better:
                self.current_value = min_value
            else:
                self.current_value = max_value

        self.version_length = version_length
        self.version = self.score_to_version()

    def __same_type(self, other):
        are_same_type = (self.name == other.name)
        are_same_type = (are_same_type and (
            self.abreviation == other.abreviation))
        are_same_type = (are_same_type and (self.min_value == other.min_value))
        are_same_type = (are_same_type and (self.max_value == other.max_value))
        are_same_type = (are_same_type and (
            self.is_bigger_better == other.is_bigger_better))
        return(are_same_type)

    def __eq__(self, other):
        are_equal = False
        if self.__same_type(other):
            are_equal = (self.current_value == other.current_value)
        else:
            raise "Error Performance: Different Metric Types"
        return(are_equal)

    def __lt__(self, other):
        is_less = False
        if self.__same_type(other):
            if self.is_bigger_better:
                is_less = (self.current_value < other.current_value)
            else:
                is_less = (self.current_value > other.current_value)
        else:
            raise "Error Performance: Different Metric Types"
        return(is_less)

    def __le__(self, other):
        return(self.__eq__(other) or self.__lt__(other))

    def __gt__(self, other):
        return(not self.__le__(other))

    def __ge__(self, other):
        return(not self.__lt__(other))

    def set_current_value(self, *args, **kwargs):
        self.current_value = self.metric_function(*args, **kwargs)
        self.version = self.score_to_version()

    def get_main_description(self):
        desc = "Metric:        {}\n".format(self.name)
        desc += "Abreviation:   {}\n".format(self.abreviation)
        desc += "Current Value: {}\n".format(self.current_value)
        desc += "Version:       {}\n".format(self.version)
        return(desc)

    def score_to_version(self):
        ver_val = 0.0
        # if self.current_value is not None:
        assert ((self.min_value <= self.current_value) and (
            self.current_value <= self.max_value)), "Score limits off bounds"

        if self.current_value == self.min_value:
            ver_val = 0.0
        elif self.current_value == self.max_value:
            ver_val = 1.0
        else:
            # Begin mapping intervals [min_value, max_value]
            # to interval [0.0, 1.0]
            epsilon = 0.1  # auxiliary value for transformations
            if self.min_value > -1.0*float("Inf") and self.max_value < float("Inf"):
                ver_val = (self.current_value - self.min_value) / \
                    (self.max_value - self.min_value)
            elif self.min_value == -1.0*float("Inf") and self.max_value < float("Inf"):
                ver_val = epsilon / \
                    (self.max_value - self.current_value + epsilon)
            elif self.min_value > -1.0*float("Inf") and self.max_value == float("Inf"):
                ver_val = (self.current_value - self.min_value) / \
                    (self.current_value - self.min_value + epsilon)
            # This case is added just for logical completion
            else:
                if self.current_value < 0:
                    ver_val = 1.0 / (2.0 - self.current_value)
                else:
                    ver_val = (self.current_value + 1.0) / \
                        (self.current_value + 2.0)

        # If lower value is better, must invert interval
        if not self.is_bigger_better:
            ver_val = -1.0*ver_val + 1.0

        # We take 0.99999... as the maximum 
        # version for tracking purposes
        if ver_val == 1.0:
            ver_val = 1.0 - 1.0 / (10**(self.version_length))

        # Recover <ver_length> decimals to add to version
        # ret = "%.{}f".format(self.version_length) % ver_val
        ret = str(int(ver_val * (10**self.version_length)))

        return(ret)
